import time
from threading import Thread

CORE_COUNT = 4 #This is how many CPUs or cores we have to play with.

def xmap(fn, seq):
  """
  Run a function over a sequence, and return the results.
  The workload is split over multiple threads.
  """
  class Mapper(Thread):
      def __init__(self, fn, seq):
          Thread.__init__(self, target=self.map, args=(fn,seq))
          self.start()
        
      def map(self, fn, seq):
          self.results = list(map(fn, seq))

  newseq = []
  n = len(seq) / CORE_COUNT
  r = len(seq) % CORE_COUNT
  b,e = 0, int(n + min(1, r))
  for i in range(CORE_COUNT):
      newseq.append(seq[b:e])
      r = max(0, r-1)
      b,e = e, int(e + n + min(1, r))

  results = []
  for thread in [Mapper(fn,s) for s in newseq]:
      thread.join()
      results.extend(thread.results)
  return results

if __name__ == "__main__":

  def double(x): return x * 2

  input = []
  for i in range(10000000):
    input.append(i)

  start = time.time()
  list(map(double, input)) 
  print("Single threaded map took: %f seconds" % (time.time() - start))
  start = time.time()
  xmap(double, input)
  print("Multi-threaded map took: %f seconds" % (time.time() - start))
